Package: filehash
Version: 2.4-6
Depends: R (>= 3.0.0)
Imports: digest, methods
Collate: filehash.R filehash-DB1.R filehash-RDS.R coerce.R dump.R
        hash.R queue.R stack.R zzz.R
Title: Simple Key-Value Database
Author: Roger D. Peng <roger.peng@austin.utexas.edu>
Maintainer: Roger D. Peng <roger.peng@austin.utexas.edu>
Description: Implements a simple key-value style database where character string keys
  are associated with data values that are stored on the disk. A simple interface is provided for inserting,
  retrieving, and deleting data from the database. Utilities are provided that allow 'filehash' databases to be
  treated much like environments and lists are already used in R. These utilities are provided to encourage
  interactive and exploratory analysis on large datasets. Three different file formats for representing the
  database are currently available and new formats can easily be incorporated by third parties for use in the
  'filehash' framework.
License: GPL (>= 2)
URL: https://github.com/rdpeng/filehash
RoxygenNote: 7.3.1
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2024-06-25 13:57:00 UTC; rp34949
Repository: CRAN
Date/Publication: 2024-06-25 21:20:10 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 16:40:38 UTC; unix
Archs: filehash.so.dSYM
