######################################################################
## Copyright (C) 2006, Roger D. Peng <rpeng@jhsph.edu>
##     
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
## 02110-1301, USA
#####################################################################

toDBType <- function(from, type, dbpath = NULL) {
    if(is.null(dbpath))
        dbpath <- dbName(from)
    if(!dbCreate(dbpath, type = type))
        stop("could not create ", type, " database")
    db <- dbInit(dbpath, type = type)
    keys <- dbList(from)
    
    for(key in keys)
        dbInsert(db, key, dbFetch(from, key))
    invisible(db)
}

setAs("filehashDB1", "filehashRDS",
      function(from) {
          dbpath <- paste(dbName(from), "RDS", sep = "")
          toDBType(from, "RDS", dbpath)
      })
      
setAs("filehashDB1", "list",
      function(from) {
          keys <- dbList(from)
          dbMultiFetch(from, keys)
      })
